import abc


class BasePlugin(abc.ABC):
    '''
    BasePlugin is the simplest base class that a Plugin can inherit from.
    It's the class you should use if your plugin is very simple or if you
    want total freedom on the design of your Plugin subclass.

    If you need to write a plugin that acts on dynamically selected
    representations, adaptationSets or periods, you should have a look
    at ManifestIteratorPlugin base class instead.

    Plugins are supposed to provide at least one amongst the four possible
    processing methods, that are:
    
    - def mpd(self, manifest, storage)
    - def m3u8(self, manifest, storage):
    - def Manifest(self, manifest, storage):
    - def f4m(self, manifest, storage):

    At runtime though, each instance of a Plugin class will be specialized in
    handling one format only. This is accomplished at plugin creation time
    by dynamically adding a "process" method pointing to the specific format
    processing one.
    '''

    # For every manifest format in the .yaml file, there will be a copy of
    # the plugin having the "process" function specialized for the particular
    # format.
    def onCreate(self, manifest_format, config):
        '''
        This method is invoked automatically at pipeline creation time and
        creats a "process" method pointing to the correct processing
        method for the specific manifest_format.
        '''
        assert (hasattr(self, manifest_format))
        self.process = getattr(self, manifest_format)
        self._config = config

    # Plugins MUST reimplement at least these methods to be valid

    # This is invoked at creation time and is necessary to validate
    #  that the .yaml configuration provided for the specific plugin
    #  is valid. The recommended way to implement this is using Schema.
    # Doing it here has the advantages of having the actual manifest processing
    #  code (i.e mpd()) cleaner (you can give for granted that your config
    #  dict will have the expected keys,values). Plus, configuration validation
    #  can be computationally expensive so it is just right to do it once
    #  when the .yaml config file is loaded.

    def config(self, storage):
        # The "config" method of a BasePlugin will return a dictionary type
        if self._config is not None:
            return self._config
        else:
            # Plugins will pass a configuration down the pipeling by putting it
            #  into a {<plugin name> : {<config>}} dictionary. The split is
            #  there to remove the path. They would use the name of the plugin
            #  that must receive the configuration.
            return storage.get(self._name.split('.')[-1], {})

    @property
    @abc.abstractmethod
    def schema(self):
        raise NotImplementedError

    def validate_config(self):
        self._config = self.schema().validate(self._config)
        return self._config

    # TODO: the right thing is to find a way to have Plugin class
    #  reimplement at least one amongst the 4 possible entrypoint.
    # Their name should be taken from the 4 supported formats
    #  that are now defined in pipeline.py
    @abc.abstractmethod
    def mpd(self, manifest, storage):
        raise NotImplementedError
