from .base_plugin import BasePlugin
from .manifest_iterators import ManifestIterator


class ManifestIteratorPlugin(BasePlugin):
    '''
    A ManifestIterator Plugin will have a slightly more complicated config
    that is a list of tuples

    .. code-block:: python
    
        [
            (
                {<specific plugin "how" config>},
                <manifest element to apply the config to>
            )
        ]

    '''

    # This is an override of the BasePlugin method
    def validate_config(self):
        # This is necessary to validate the configuration coming from the yml
        #  file once at loading time.
        # If the configuration is empty, it means this plugin will receive
        #  its configuration at runtime in the "storage" argument of the mpd
        #  method, so there is nothing to check.
        if self._config:
            self.manifest_iterator = ManifestIterator(
                self._config, plugin_config_schema=self.schema())
            return self.manifest_iterator.is_valid()
        else:
            return True

    # This is an override of the BasePlugin method
    def config(self, manifest, storage, keys=None):
        if keys is None:
            keys = self._keys
        # The "config" method of a ManifestIteratorPlugin Type will use
        # either an iterator coming from ManifestIterator or will iterate
        # on the config passed from the previous plugin and intended to be
        # used by this plugin.
        if self._config is not None:
            # This means the yml config was valid and we have to use the
            #  internal ManifestIterator
            yield from self.manifest_iterator(
                manifest,
                keys
            )
        else:
            # Plugins will pass a configuration down the pipeling by putting it
            #  into a {<plugin name> : {<config>}} dictionary. The split is
            #  there to remove the path. They would use the name of the plugin
            #  that must receive the configuration.
            yield from storage.get(self._name.split('.')[-1], [])
